@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
@@ -16,25 +16,21 @@
 
 #include <cm/optional>
 
-#include "cm_codecvt.hxx"
+#include "cm_codecvt_Encoding.hxx"
 
+#include "cmBuildOptions.h"
 #include "cmGeneratedFileStream.h"
 #include "cmGlobalCommonGenerator.h"
 #include "cmGlobalGeneratorFactory.h"
 #include "cmNinjaTypes.h"
-#include "cmPolicies.h"
 #include "cmStringAlgorithms.h"
 #include "cmTransformDepfile.h"
 
 class cmCustomCommand;
 class cmGeneratorTarget;
-class cmLinkLineComputer;
-class cmLocalGenerator;
 class cmMakefile;
-class cmOutputConverter;
-class cmStateDirectory;
 class cmake;
-struct cmDocumentationEntry;
+struct cmCxxModuleExportInfo;
 
 /**
  * \class cmGlobalNinjaGenerator
@@ -57,14 +53,14 @@ class cmGlobalNinjaGenerator : public cmGlobalCommonGenerator
 {
 public:
   /// The default name of Ninja's build file. Typically: build.ninja.
-  static const char* NINJA_BUILD_FILE;
+  static char const* NINJA_BUILD_FILE;
 
   /// The default name of Ninja's rules file. Typically: rules.ninja.
   /// It is included in the main build.ninja file.
-  static const char* NINJA_RULES_FILE;
+  static char const* NINJA_RULES_FILE;
 
   /// The indentation string used when generating Ninja's build file.
-  static const char* INDENT;
+  static char const* INDENT;
 
   /// The shell command used for a no-op.
   static std::string const SHELL_NOOP;
@@ -76,8 +72,8 @@ public:
   static void WriteDivider(std::ostream& os);
 
   static std::string EncodeRuleName(std::string const& name);
-  std::string EncodeLiteral(const std::string& lit);
-  std::string EncodePath(const std::string& path);
+  std::string& EncodeLiteral(std::string& lit) override;
+  std::string EncodePath(std::string const& path);
 
   std::unique_ptr<cmLinkLineComputer> CreateLinkLineComputer(
     cmOutputConverter* outputConverter,
@@ -87,7 +83,7 @@ public:
    * Write the given @a comment to the output stream @a os. It
    * handles new line character properly.
    */
-  static void WriteComment(std::ostream& os, const std::string& comment);
+  static void WriteComment(std::ostream& os, std::string const& comment);
 
   /**
    * Utilized by the generator factory to determine if this generator
@@ -135,7 +131,7 @@ public:
                                cmNinjaDeps orderOnlyDeps = cmNinjaDeps());
 
   void WriteMacOSXContentBuild(std::string input, std::string output,
-                               const std::string& config);
+                               std::string const& config);
 
   /**
    * Write a rule statement to @a os.
@@ -148,25 +144,26 @@ public:
    * optional @a comment. An @a indent level can be specified.
    * @warning no escaping of any kind is done here.
    */
-  static void WriteVariable(std::ostream& os, const std::string& name,
-                            const std::string& value,
-                            const std::string& comment = "", int indent = 0);
+  static void WriteVariable(std::ostream& os, std::string const& name,
+                            std::string const& value,
+                            std::string const& comment = "", int indent = 0);
 
   /**
    * Write an include statement including @a filename with an optional
    * @a comment to the @a os stream.
    */
-  static void WriteInclude(std::ostream& os, const std::string& filename,
-                           const std::string& comment = "");
+  static void WriteInclude(std::ostream& os, std::string const& filename,
+                           std::string const& comment = "");
 
   /**
    * Write a default target statement specifying @a targets as
    * the default targets.
    */
-  static void WriteDefault(std::ostream& os, const cmNinjaDeps& targets,
-                           const std::string& comment = "");
+  static void WriteDefault(std::ostream& os, cmNinjaDeps const& targets,
+                           std::string const& comment = "");
 
   bool IsGCCOnWindows() const { return this->UsingGCCOnWindows; }
+  void MarkAsGCCOnWindows() { this->UsingGCCOnWindows = true; }
 
   cmGlobalNinjaGenerator(cmake* cm);
 
@@ -189,43 +186,47 @@ public:
   bool IsNinja() const override { return true; }
 
   /** Get encoding used by generator for ninja files */
-  codecvt::Encoding GetMakefileEncoding() const override;
+  codecvt_Encoding GetMakefileEncoding() const override;
 
-  static void GetDocumentation(cmDocumentationEntry& entry);
+  static cmDocumentationEntry GetDocumentation();
 
   void EnableLanguage(std::vector<std::string> const& languages,
                       cmMakefile* mf, bool optional) override;
 
   std::vector<GeneratedMakeCommand> GenerateBuildCommand(
-    const std::string& makeProgram, const std::string& projectName,
-    const std::string& projectDir, std::vector<std::string> const& targetNames,
-    const std::string& config, bool fast, int jobs, bool verbose,
-    std::vector<std::string> const& makeOptions =
-      std::vector<std::string>()) override;
+    std::string const& makeProgram, std::string const& projectName,
+    std::string const& projectDir, std::vector<std::string> const& targetNames,
+    std::string const& config, int jobs, bool verbose,
+    cmBuildOptions buildOptions = cmBuildOptions(),
+    std::vector<std::string> const& makeOptions = std::vector<std::string>(),
+    BuildTryCompile isInTryCompile = BuildTryCompile::No) override;
 
   // Setup target names
-  const char* GetAllTargetName() const override { return "all"; }
-  const char* GetInstallTargetName() const override { return "install"; }
-  const char* GetInstallLocalTargetName() const override
+  char const* GetAllTargetName() const override { return "all"; }
+  char const* GetInstallTargetName() const override { return "install"; }
+  char const* GetInstallLocalTargetName() const override
   {
     return "install/local";
   }
-  const char* GetInstallStripTargetName() const override
+  char const* GetInstallStripTargetName() const override
   {
     return "install/strip";
   }
-  const char* GetTestTargetName() const override { return "test"; }
-  const char* GetPackageTargetName() const override { return "package"; }
-  const char* GetPackageSourceTargetName() const override
+  char const* GetInstallParallelTargetName() const
+  {
+    return "install/parallel";
+  }
+  char const* GetTestTargetName() const override { return "test"; }
+  char const* GetPackageTargetName() const override { return "package"; }
+  char const* GetPackageSourceTargetName() const override
   {
     return "package_source";
   }
-  const char* GetEditCacheTargetName() const override { return "edit_cache"; }
-  const char* GetRebuildCacheTargetName() const override
+  char const* GetRebuildCacheTargetName() const override
   {
     return "rebuild_cache";
   }
-  const char* GetCleanTargetName() const override { return "clean"; }
+  char const* GetCleanTargetName() const override { return "clean"; }
 
   bool SupportsCustomCommandDepfile() const override { return true; }
   cm::optional<cmDepfileFormat> DepfileFormat() const override
@@ -233,14 +234,16 @@ public:
     return cmDepfileFormat::GccDepfile;
   }
 
+  bool SupportsLinkerDependencyFile() const override { return true; }
+
   virtual cmGeneratedFileStream* GetImplFileStream(
-    const std::string& /*config*/) const
+    std::string const& /*config*/) const
   {
     return this->BuildFileStream.get();
   }
 
   virtual cmGeneratedFileStream* GetConfigFileStream(
-    const std::string& /*config*/) const
+    std::string const& /*config*/) const
   {
     return this->BuildFileStream.get();
   }
@@ -260,7 +263,7 @@ public:
     return this->RulesFileStream.get();
   }
 
-  std::string const& ConvertToNinjaPath(const std::string& path) const;
+  std::string const& ConvertToNinjaPath(std::string const& path) const;
   std::string ConvertToNinjaAbsPath(std::string path) const;
 
   struct MapToNinjaPathImpl
@@ -277,20 +280,25 @@ public:
   };
   MapToNinjaPathImpl MapToNinjaPath() { return { this }; }
 
+#ifdef _WIN32
+  std::string const& GetComspec() const { return this->Comspec; }
+#endif
+
   // -- Additional clean files
-  void AddAdditionalCleanFile(std::string fileName, const std::string& config);
-  const char* GetAdditionalCleanTargetName() const
+  void AddAdditionalCleanFile(std::string fileName, std::string const& config);
+  char const* GetAdditionalCleanTargetName() const
   {
     return "CMakeFiles/clean.additional";
   }
 
-  static const char* GetByproductsForCleanTargetName()
+  static char const* GetByproductsForCleanTargetName()
   {
     return "CMakeFiles/cmake_byproducts_for_clean_target";
   }
 
-  void AddCXXCompileCommand(const std::string& commandLine,
-                            const std::string& sourceFile);
+  void AddCXXCompileCommand(std::string const& commandLine,
+                            std::string const& sourceFile,
+                            std::string const& objPath);
 
   /**
    * Add a rule to the generated build system.
@@ -299,12 +307,12 @@ public:
    */
   void AddRule(cmNinjaRule const& rule);
 
-  bool HasRule(const std::string& name);
+  bool HasRule(std::string const& name);
 
   void AddCustomCommandRule();
   void AddMacOSXContentRule();
 
-  bool HasCustomCommandOutput(const std::string& output)
+  bool HasCustomCommandOutput(std::string const& output)
   {
     return this->CustomCommandOutputs.find(output) !=
       this->CustomCommandOutputs.end();
@@ -312,13 +320,13 @@ public:
 
   /// Called when we have seen the given custom command.  Returns true
   /// if we has seen it before.
-  bool SeenCustomCommand(cmCustomCommand const* cc, const std::string& config)
+  bool SeenCustomCommand(cmCustomCommand const* cc, std::string const& config)
   {
     return !this->Configs[config].CustomCommands.insert(cc).second;
   }
 
   /// Called when we have seen the given custom command output.
-  void SeenCustomCommandOutput(const std::string& output)
+  void SeenCustomCommandOutput(std::string const& output)
   {
     this->CustomCommandOutputs.insert(output);
     // We don't need the assumed dependencies anymore, because we have
@@ -326,8 +334,8 @@ public:
     this->AssumedSourceDependencies.erase(output);
   }
 
-  void AddAssumedSourceDependencies(const std::string& source,
-                                    const cmNinjaDeps& deps)
+  void AddAssumedSourceDependencies(std::string const& source,
+                                    cmNinjaDeps const& deps)
   {
     std::set<std::string>& ASD = this->AssumedSourceDependencies[source];
     // Because we may see the same source file multiple times (same source
@@ -337,49 +345,49 @@ public:
   }
 
   virtual std::string OrderDependsTargetForTarget(
-    cmGeneratorTarget const* target, const std::string& config) const;
+    cmGeneratorTarget const* target, std::string const& config) const;
+
+  std::string OrderDependsTargetForTargetPrivate(
+    cmGeneratorTarget const* target, std::string const& config) const;
 
   void AppendTargetOutputs(cmGeneratorTarget const* target,
-                           cmNinjaDeps& outputs, const std::string& config,
+                           cmNinjaDeps& outputs, std::string const& config,
                            cmNinjaTargetDepends depends) const;
   void AppendTargetDepends(cmGeneratorTarget const* target,
-                           cmNinjaDeps& outputs, const std::string& config,
-                           const std::string& fileConfig,
+                           cmNinjaDeps& outputs, std::string const& config,
+                           std::string const& fileConfig,
                            cmNinjaTargetDepends depends);
   void AppendTargetDependsClosure(cmGeneratorTarget const* target,
-                                  cmNinjaDeps& outputs,
-                                  const std::string& config,
-                                  const std::string& fileConfig,
-                                  bool genexOutput);
-  void AppendTargetDependsClosure(cmGeneratorTarget const* target,
-                                  cmNinjaOuts& outputs,
-                                  const std::string& config,
-                                  const std::string& fileConfig,
-                                  bool genexOutput, bool omit_self);
+                                  std::unordered_set<std::string>& outputs,
+                                  std::string const& config,
+                                  std::string const& fileConfig,
+                                  bool genexOutput, bool omit_self = true);
 
-  void AppendDirectoryForConfig(const std::string& prefix,
-                                const std::string& config,
-                                const std::string& suffix,
+  void AppendDirectoryForConfig(std::string const& prefix,
+                                std::string const& config,
+                                std::string const& suffix,
                                 std::string& dir) override;
 
   virtual void AppendNinjaFileArgument(GeneratedMakeCommand& /*command*/,
-                                       const std::string& /*config*/) const
+                                       std::string const& /*config*/) const
   {
   }
 
   virtual void AddRebuildManifestOutputs(cmNinjaDeps& outputs) const
   {
     outputs.push_back(this->NinjaOutputPath(NINJA_BUILD_FILE));
+    this->AddCMakeFilesToRebuild(outputs);
   }
 
-  int GetRuleCmdLength(const std::string& name)
+  int GetRuleCmdLength(std::string const& name)
   {
     return this->RuleCmdLength[name];
   }
 
-  void AddTargetAlias(const std::string& alias, cmGeneratorTarget* target,
-                      const std::string& config);
+  void AddTargetAlias(std::string const& alias, cmGeneratorTarget* target,
+                      std::string const& config);
 
+  bool SupportsShortObjectNames() const override;
   void ComputeTargetObjectDirectory(cmGeneratorTarget* gt) const override;
 
   // Ninja generator uses 'deps' and 'msvc_deps_prefix' introduced in 1.3
@@ -391,7 +399,8 @@ public:
   {
     return "1.9";
   }
-  static std::string RequiredNinjaVersionForDyndeps() { return "1.10"; }
+  static std::string RequiredNinjaVersionForDyndepsCxx() { return "1.11"; }
+  static std::string RequiredNinjaVersionForDyndepsFortran() { return "1.10"; }
   static std::string RequiredNinjaVersionForRestatTool() { return "1.10"; }
   static std::string RequiredNinjaVersionForUnconditionalRecompactTool()
   {
@@ -406,10 +415,12 @@ public:
     return "1.10.2";
   }
   static std::string RequiredNinjaVersionForCodePage() { return "1.11"; }
-  bool SupportsConsolePool() const;
+  static std::string RequiredNinjaVersionForCWDDepend() { return "1.7"; }
+  bool SupportsDirectConsole() const override;
   bool SupportsImplicitOuts() const;
   bool SupportsManifestRestat() const;
   bool SupportsMultilineDepfile() const;
+  bool SupportsCWDDepend() const;
 
   std::string NinjaOutputPath(std::string const& path) const;
   bool HasOutputPathPrefix() const { return !this->OutputPathPrefix.empty(); }
@@ -421,15 +432,17 @@ public:
     std::string const& arg_dd, std::vector<std::string> const& arg_ddis,
     std::string const& module_dir,
     std::vector<std::string> const& linked_target_dirs,
-    std::string const& arg_lang, std::string const& arg_modmapfmt);
+    std::vector<std::string> const& forward_modules_from_target_dirs,
+    std::string const& arg_lang, std::string const& arg_modmapfmt,
+    cmCxxModuleExportInfo const& export_info);
 
-  virtual std::string BuildAlias(const std::string& alias,
-                                 const std::string& /*config*/) const
+  virtual std::string BuildAlias(std::string const& alias,
+                                 std::string const& /*config*/) const
   {
     return alias;
   }
 
-  virtual std::string ConfigDirectory(const std::string& /*config*/) const
+  virtual std::string ConfigDirectory(std::string const& /*config*/) const
   {
     return "";
   }
@@ -439,35 +452,40 @@ public:
     return this->ByproductsForCleanTarget;
   }
 
-  cmNinjaDeps& GetByproductsForCleanTarget(const std::string& config)
+  cmNinjaDeps& GetByproductsForCleanTarget(std::string const& config)
   {
     return this->Configs[config].ByproductsForCleanTarget;
   }
 
   bool EnableCrossConfigBuild() const;
 
-  std::set<std::string> GetCrossConfigs(const std::string& config) const;
+  std::set<std::string> GetCrossConfigs(std::string const& config) const;
 
-  const std::set<std::string>& GetDefaultConfigs() const
+  std::set<std::string> const& GetDefaultConfigs() const override
   {
     return this->DefaultConfigs;
   }
 
-  const std::set<std::string>& GetPerConfigUtilityTargets() const
+  std::set<std::string> const& GetPerConfigUtilityTargets() const
   {
     return this->PerConfigUtilityTargets;
   }
 
-  void AddPerConfigUtilityTarget(const std::string& name)
+  void AddPerConfigUtilityTarget(std::string const& name)
   {
     this->PerConfigUtilityTargets.insert(name);
   }
 
   bool IsSingleConfigUtility(cmGeneratorTarget const* target) const;
 
-  bool CheckCxxModuleSupport();
+  bool CheckCxxModuleSupport(CxxModuleSupportQuery query) override;
+  bool SupportsBuildDatabase() const override { return true; }
+
+  std::string ConvertToOutputPath(std::string path) const override;
 
 protected:
+  std::vector<std::string> const& GetConfigNames() const;
+
   void Generate() override;
 
   bool CheckALLOW_DUPLICATE_CUSTOM_TARGETS() const override { return true; }
@@ -476,20 +494,17 @@ protected:
   virtual void CloseBuildFileStreams();
 
   bool OpenFileStream(std::unique_ptr<cmGeneratedFileStream>& stream,
-                      const std::string& name);
+                      std::string const& name);
 
   static cm::optional<std::set<std::string>> ListSubsetWithAll(
-    const std::set<std::string>& all, const std::set<std::string>& defaults,
-    const std::vector<std::string>& items);
-
-  virtual bool InspectConfigTypeVariables() { return true; }
+    std::set<std::string> const& all, std::set<std::string> const& defaults,
+    std::vector<std::string> const& items);
 
   std::set<std::string> CrossConfigs;
   std::set<std::string> DefaultConfigs;
   std::string DefaultFileConfig;
 
 private:
-  std::string GetEditCacheCommand() const override;
   bool FindMakeProgram(cmMakefile* mf) override;
   void CheckNinjaFeatures();
   void CheckNinjaCodePage();
@@ -511,13 +526,15 @@ private:
 
   void WriteTargetAliases(std::ostream& os);
   void WriteFolderTargets(std::ostream& os);
-  void WriteUnknownExplicitDependencies(std::ostream& os);
 
   void WriteBuiltinTargets(std::ostream& os);
   void WriteTargetDefault(std::ostream& os);
   void WriteTargetRebuildManifest(std::ostream& os);
   bool WriteTargetCleanAdditional(std::ostream& os);
   void WriteTargetClean(std::ostream& os);
+#ifndef CMAKE_BOOTSTRAP
+  void WriteTargetInstrument(std::ostream& os);
+#endif
   void WriteTargetHelp(std::ostream& os);
 
   void ComputeTargetDependsClosure(
@@ -546,18 +563,6 @@ private:
   /// The set of custom command outputs we have seen.
   std::set<std::string> CustomCommandOutputs;
 
-  /// Whether we are collecting known build outputs and needed
-  /// dependencies to determine unknown dependencies.
-  bool ComputingUnknownDependencies = false;
-  cmPolicies::PolicyStatus PolicyCMP0058 = cmPolicies::WARN;
-
-  /// The combined explicit dependencies of custom build commands
-  std::set<std::string> CombinedCustomCommandExplicitDependencies;
-
-  /// When combined with CombinedCustomCommandExplicitDependencies it allows
-  /// us to detect the set of explicit dependencies that have
-  std::set<std::string> CombinedBuildOutputs;
-
   /// The mapping from source file to assumed dependencies.
   std::map<std::string, std::set<std::string>> AssumedSourceDependencies;
 
@@ -582,23 +587,29 @@ private:
   bool NinjaSupportsImplicitOuts = false;
   bool NinjaSupportsManifestRestat = false;
   bool NinjaSupportsMultilineDepfile = false;
-  bool NinjaSupportsDyndeps = false;
+  bool NinjaSupportsDyndepsCxx = false;
+  bool NinjaSupportsDyndepsFortran = false;
   bool NinjaSupportsRestatTool = false;
   bool NinjaSupportsUnconditionalRecompactTool = false;
   bool NinjaSupportsMultipleOutputs = false;
   bool NinjaSupportsMetadataOnRegeneration = false;
   bool NinjaSupportsCodePage = false;
+  bool NinjaSupportsCWDDepend = false;
 
-  codecvt::Encoding NinjaExpectedEncoding = codecvt::None;
+  codecvt_Encoding NinjaExpectedEncoding = codecvt_Encoding::None;
 
-  bool DiagnosedCxxModuleSupport = false;
+#ifdef _WIN32
+  // Windows Command shell.
+  std::string Comspec;
+#endif
+
+  bool DiagnosedCxxModuleNinjaSupport = false;
 
   void InitOutputPathPrefix();
 
   std::string OutputPathPrefix;
   std::string TargetAll;
   std::string CMakeCacheFile;
-  bool DisableCleandead = false;
 
   struct ByConfig
   {
@@ -614,7 +625,8 @@ private:
       bool GenexOutput;
     };
 
-    std::map<TargetDependsClosureKey, cmNinjaOuts> TargetDependsClosures;
+    std::map<TargetDependsClosureKey, std::unordered_set<std::string>>
+      TargetDependsClosures;
 
     TargetAliasMap TargetAliases;
 
@@ -624,27 +636,27 @@ private:
 
   cmNinjaDeps ByproductsForCleanTarget;
 
-  friend bool operator==(const ByConfig::TargetDependsClosureKey& lhs,
-                         const ByConfig::TargetDependsClosureKey& rhs);
-  friend bool operator!=(const ByConfig::TargetDependsClosureKey& lhs,
-                         const ByConfig::TargetDependsClosureKey& rhs);
-  friend bool operator<(const ByConfig::TargetDependsClosureKey& lhs,
-                        const ByConfig::TargetDependsClosureKey& rhs);
-  friend bool operator>(const ByConfig::TargetDependsClosureKey& lhs,
-                        const ByConfig::TargetDependsClosureKey& rhs);
-  friend bool operator<=(const ByConfig::TargetDependsClosureKey& lhs,
-                         const ByConfig::TargetDependsClosureKey& rhs);
-  friend bool operator>=(const ByConfig::TargetDependsClosureKey& lhs,
-                         const ByConfig::TargetDependsClosureKey& rhs);
+  friend bool operator==(ByConfig::TargetDependsClosureKey const& lhs,
+                         ByConfig::TargetDependsClosureKey const& rhs);
+  friend bool operator!=(ByConfig::TargetDependsClosureKey const& lhs,
+                         ByConfig::TargetDependsClosureKey const& rhs);
+  friend bool operator<(ByConfig::TargetDependsClosureKey const& lhs,
+                        ByConfig::TargetDependsClosureKey const& rhs);
+  friend bool operator>(ByConfig::TargetDependsClosureKey const& lhs,
+                        ByConfig::TargetDependsClosureKey const& rhs);
+  friend bool operator<=(ByConfig::TargetDependsClosureKey const& lhs,
+                         ByConfig::TargetDependsClosureKey const& rhs);
+  friend bool operator>=(ByConfig::TargetDependsClosureKey const& lhs,
+                         ByConfig::TargetDependsClosureKey const& rhs);
 };
 
 class cmGlobalNinjaMultiGenerator : public cmGlobalNinjaGenerator
 {
 public:
   /// The default name of Ninja's common file. Typically: common.ninja.
-  static const char* NINJA_COMMON_FILE;
+  static char const* NINJA_COMMON_FILE;
   /// The default file extension to use for per-config Ninja files.
-  static const char* NINJA_FILE_EXTENSION;
+  static char const* NINJA_FILE_EXTENSION;
 
   cmGlobalNinjaMultiGenerator(cmake* cm);
   bool IsMultiConfig() const override { return true; }
@@ -654,7 +666,7 @@ public:
       new cmGlobalGeneratorSimpleFactory<cmGlobalNinjaMultiGenerator>());
   }
 
-  static void GetDocumentation(cmDocumentationEntry& entry);
+  static cmDocumentationEntry GetDocumentation();
 
   std::string GetName() const override
   {
@@ -663,8 +675,8 @@ public:
 
   static std::string GetActualName() { return "Ninja Multi-Config"; }
 
-  std::string BuildAlias(const std::string& alias,
-                         const std::string& config) const override
+  std::string BuildAlias(std::string const& alias,
+                         std::string const& config) const override
   {
     if (config.empty()) {
       return alias;
@@ -672,7 +684,7 @@ public:
     return cmStrCat(alias, ":", config);
   }
 
-  std::string ConfigDirectory(const std::string& config) const override
+  std::string ConfigDirectory(std::string const& config) const override
   {
     if (!config.empty()) {
       return cmStrCat('/', config);
@@ -680,19 +692,19 @@ public:
     return "";
   }
 
-  const char* GetCMakeCFGIntDir() const override { return "${CONFIGURATION}"; }
+  char const* GetCMakeCFGIntDir() const override { return "${CONFIGURATION}"; }
 
-  std::string ExpandCFGIntDir(const std::string& str,
-                              const std::string& config) const override;
+  std::string ExpandCFGIntDir(std::string const& str,
+                              std::string const& config) const override;
 
   cmGeneratedFileStream* GetImplFileStream(
-    const std::string& config) const override
+    std::string const& config) const override
   {
     return this->ImplFileStreams.at(config).get();
   }
 
   cmGeneratedFileStream* GetConfigFileStream(
-    const std::string& config) const override
+    std::string const& config) const override
   {
     return this->ConfigFileStreams.at(config).get();
   }
@@ -708,10 +720,10 @@ public:
   }
 
   void AppendNinjaFileArgument(GeneratedMakeCommand& command,
-                               const std::string& config) const override;
+                               std::string const& config) const override;
 
-  static std::string GetNinjaImplFilename(const std::string& config);
-  static std::string GetNinjaConfigFilename(const std::string& config);
+  static std::string GetNinjaImplFilename(std::string const& config);
+  static std::string GetNinjaConfigFilename(std::string const& config);
 
   void AddRebuildManifestOutputs(cmNinjaDeps& outputs) const override;
 
@@ -726,7 +738,7 @@ public:
   bool SupportsDefaultConfigs() const override { return true; }
 
   std::string OrderDependsTargetForTarget(
-    cmGeneratorTarget const* target, const std::string& config) const override;
+    cmGeneratorTarget const* target, std::string const& config) const override;
 
 protected:
   bool OpenBuildFileStreams() override;
