@@ -1,15 +1,13 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 // NOLINTNEXTLINE(bugprone-reserved-identifier)
 #define _SCL_SECURE_NO_WARNINGS
 
 #include "cmStringCommand.h"
 
 #include <algorithm>
-#include <cctype>
 #include <cstdio>
 #include <cstdlib>
-#include <initializer_list>
 #include <limits>
 #include <memory>
 #include <stdexcept>
@@ -31,7 +29,7 @@
 #include "cmGeneratorExpression.h"
 #include "cmMakefile.h"
 #include "cmMessageType.h"
-#include "cmProperty.h"
+#include "cmPolicies.h"
 #include "cmRange.h"
 #include "cmStringAlgorithms.h"
 #include "cmStringReplaceHelper.h"
@@ -39,6 +37,7 @@
 #include "cmSystemTools.h"
 #include "cmTimestamp.h"
 #include "cmUuid.h"
+#include "cmValue.h"
 
 namespace {
 
@@ -48,6 +47,8 @@ bool RegexMatchAll(std::vector<std::string> const& args,
                    cmExecutionStatus& status);
 bool RegexReplace(std::vector<std::string> const& args,
                   cmExecutionStatus& status);
+bool RegexQuote(std::vector<std::string> const& args,
+                cmExecutionStatus& status);
 
 bool joinImpl(std::vector<std::string> const& args, std::string const& glue,
               size_t varIdx, cmMakefile& makefile);
@@ -55,7 +56,6 @@ bool joinImpl(std::vector<std::string> const& args, std::string const& glue,
 bool HandleHashCommand(std::vector<std::string> const& args,
                        cmExecutionStatus& status)
 {
-#if !defined(CMAKE_BOOTSTRAP)
   if (args.size() != 3) {
     status.SetError(
       cmStrCat(args[0], " requires an output variable and an input string"));
@@ -69,10 +69,6 @@ bool HandleHashCommand(std::vector<std::string> const& args,
     return true;
   }
   return false;
-#else
-  status.SetError(cmStrCat(args[0], " not available during bootstrap"));
-  return false;
-#endif
 }
 
 bool HandleToUpperLowerCommand(std::vector<std::string> const& args,
@@ -148,7 +144,7 @@ bool HandleHexCommand(std::vector<std::string> const& args,
 
   std::string::size_type hexIndex = 0;
   for (auto const& c : instr) {
-    sprintf(&output[hexIndex], "%.2x", static_cast<unsigned char>(c) & 0xFF);
+    snprintf(&output[hexIndex], 3, "%.2x", c & 0xFFu);
     hexIndex += 2;
   }
 
@@ -177,7 +173,7 @@ bool HandleConfigureCommand(std::vector<std::string> const& args,
     } else if (args[i] == "ESCAPE_QUOTES") {
       escapeQuotes = true;
     } else {
-      status.SetError(cmStrCat("Unrecognized argument \"", args[i], "\""));
+      status.SetError(cmStrCat("Unrecognized argument \"", args[i], '"'));
       return false;
     }
   }
@@ -224,6 +220,14 @@ bool HandleRegexCommand(std::vector<std::string> const& args,
     }
     return RegexReplace(args, status);
   }
+  if (mode == "QUOTE") {
+    if (args.size() < 4) {
+      status.SetError("sub-command REGEX, mode QUOTE needs "
+                      "at least 4 arguments total to command.");
+      return false;
+    }
+    return RegexQuote(args, status);
+  }
 
   std::string e = "sub-command REGEX does not recognize mode " + mode;
   status.SetError(e);
@@ -256,15 +260,7 @@ bool RegexMatch(std::vector<std::string> const& args,
   std::string output;
   if (re.find(input)) {
     status.GetMakefile().StoreMatches(re);
-    std::string::size_type l = re.start();
-    std::string::size_type r = re.end();
-    if (r - l == 0) {
-      std::string e = "sub-command REGEX, mode MATCH regex \"" + regex +
-        "\" matched an empty string.";
-      status.SetError(e);
-      return false;
-    }
-    output = input.substr(l, r - l);
+    output = re.match();
   }
 
   // Store the output in the provided variable.
@@ -294,25 +290,33 @@ bool RegexMatchAll(std::vector<std::string> const& args,
   // Concatenate all the last arguments together.
   std::string input = cmJoin(cmMakeRange(args).advance(4), std::string());
 
+  unsigned optAnchor = 0;
+  if (status.GetMakefile().GetPolicyStatus(cmPolicies::CMP0186) !=
+      cmPolicies::NEW) {
+    optAnchor = cmsys::RegularExpression::BOL_AT_OFFSET;
+  }
+
   // Scan through the input for all matches.
   std::string output;
-  const char* p = input.c_str();
-  while (re.find(p)) {
+  std::string::size_type base = 0;
+  unsigned optNonEmpty = 0;
+  while (re.find(input, base, optAnchor | optNonEmpty)) {
     status.GetMakefile().ClearMatches();
     status.GetMakefile().StoreMatches(re);
-    std::string::size_type l = re.start();
-    std::string::size_type r = re.end();
-    if (r - l == 0) {
-      std::string e = "sub-command REGEX, mode MATCHALL regex \"" + regex +
-        "\" matched an empty string.";
-      status.SetError(e);
-      return false;
-    }
-    if (!output.empty()) {
+    if (!output.empty() || optNonEmpty) {
       output += ";";
     }
-    output += std::string(p + l, r - l);
-    p += r;
+    output += re.match();
+    base = re.end();
+
+    if (re.start() == input.length()) {
+      break;
+    }
+    if (re.start() == re.end()) {
+      optNonEmpty = cmsys::RegularExpression::NONEMPTY_AT_OFFSET;
+    } else {
+      optNonEmpty = 0;
+    }
   }
 
   // Store the output in the provided variable.
@@ -347,7 +351,7 @@ bool RegexReplace(std::vector<std::string> const& args,
   }
 
   // Concatenate all the last arguments together.
-  const std::string input =
+  std::string const input =
     cmJoin(cmMakeRange(args).advance(5), std::string());
   std::string output;
 
@@ -362,6 +366,28 @@ bool RegexReplace(std::vector<std::string> const& args,
   return true;
 }
 
+bool RegexQuote(std::vector<std::string> const& args,
+                cmExecutionStatus& status)
+{
+  //"STRING(REGEX QUOTE <output variable> <input> [<input>...]\n"
+  std::string const& outvar = args[2];
+  std::string const input =
+    cmJoin(cmMakeRange(args).advance(3), std::string());
+  std::string output;
+
+  // Escape all regex special characters
+  cmStringReplaceHelper replaceHelper("([][()+*^.$?|\\\\])", R"(\\\1)");
+  if (!replaceHelper.Replace(input, output)) {
+    status.SetError(
+      "sub-command REGEX, mode QUOTE: " + replaceHelper.GetError() + ".");
+    return false;
+  }
+
+  // Store the output in the provided variable.
+  status.GetMakefile().AddDefinition(outvar, output);
+  return true;
+}
+
 bool HandleFindCommand(std::vector<std::string> const& args,
                        cmExecutionStatus& status)
 {
@@ -384,9 +410,9 @@ bool HandleFindCommand(std::vector<std::string> const& args,
   }
 
   // local parameter names.
-  const std::string& sstring = args[1];
-  const std::string& schar = args[2];
-  const std::string& outvar = args[3];
+  std::string const& sstring = args[1];
+  std::string const& schar = args[2];
+  std::string const& outvar = args[3];
 
   // ensure that the user cannot accidentally specify REVERSE as a variable
   if (outvar == "REVERSE") {
@@ -432,9 +458,9 @@ bool HandleCompareCommand(std::vector<std::string> const& args,
       return false;
     }
 
-    const std::string& left = args[2];
-    const std::string& right = args[3];
-    const std::string& outvar = args[4];
+    std::string const& left = args[2];
+    std::string const& right = args[3];
+    std::string const& outvar = args[4];
     bool result;
     if (mode == "LESS") {
       result = (left < right);
@@ -470,9 +496,9 @@ bool HandleReplaceCommand(std::vector<std::string> const& args,
     return false;
   }
 
-  const std::string& matchExpression = args[1];
-  const std::string& replaceExpression = args[2];
-  const std::string& variableName = args[3];
+  std::string const& matchExpression = args[1];
+  std::string const& replaceExpression = args[2];
+  std::string const& variableName = args[3];
 
   std::string input = cmJoin(cmMakeRange(args).advance(4), std::string());
 
@@ -491,10 +517,10 @@ bool HandleSubstringCommand(std::vector<std::string> const& args,
     return false;
   }
 
-  const std::string& stringValue = args[1];
+  std::string const& stringValue = args[1];
   int begin = atoi(args[2].c_str());
   int end = atoi(args[3].c_str());
-  const std::string& variableName = args[4];
+  std::string const& variableName = args[4];
 
   size_t stringLength = stringValue.size();
   int intStringLength = static_cast<int>(stringLength);
@@ -521,12 +547,12 @@ bool HandleLengthCommand(std::vector<std::string> const& args,
     return false;
   }
 
-  const std::string& stringValue = args[1];
-  const std::string& variableName = args[2];
+  std::string const& stringValue = args[1];
+  std::string const& variableName = args[2];
 
   size_t length = stringValue.size();
   char buffer[1024];
-  sprintf(buffer, "%d", static_cast<int>(length));
+  snprintf(buffer, sizeof(buffer), "%d", static_cast<int>(length));
 
   status.GetMakefile().AddDefinition(variableName, buffer);
   return true;
@@ -569,10 +595,10 @@ bool HandlePrependCommand(std::vector<std::string> const& args,
     return true;
   }
 
-  const std::string& variable = args[1];
+  std::string const& variable = args[1];
 
   std::string value = cmJoin(cmMakeRange(args).advance(2), std::string());
-  cmProp oldValue = status.GetMakefile().GetDefinition(variable);
+  cmValue oldValue = status.GetMakefile().GetDefinition(variable);
   if (oldValue) {
     value += *oldValue;
   }
@@ -603,7 +629,7 @@ bool HandleJoinCommand(std::vector<std::string> const& args,
 }
 
 bool joinImpl(std::vector<std::string> const& args, std::string const& glue,
-              const size_t varIdx, cmMakefile& makefile)
+              size_t const varIdx, cmMakefile& makefile)
 {
   std::string const& variableName = args[varIdx];
   // NOTE Items to concat/join placed right after the variable for
@@ -622,8 +648,8 @@ bool HandleMakeCIdentifierCommand(std::vector<std::string> const& args,
     return false;
   }
 
-  const std::string& input = args[1];
-  const std::string& variableName = args[2];
+  std::string const& input = args[1];
+  std::string const& variableName = args[2];
 
   status.GetMakefile().AddDefinition(variableName,
                                      cmSystemTools::MakeCidentifier(input));
@@ -638,12 +664,12 @@ bool HandleGenexStripCommand(std::vector<std::string> const& args,
     return false;
   }
 
-  const std::string& input = args[1];
+  std::string const& input = args[1];
 
   std::string result = cmGeneratorExpression::Preprocess(
     input, cmGeneratorExpression::StripAllGeneratorExpressions);
 
-  const std::string& variableName = args[2];
+  std::string const& variableName = args[2];
 
   status.GetMakefile().AddDefinition(variableName, result);
   return true;
@@ -657,15 +683,15 @@ bool HandleStripCommand(std::vector<std::string> const& args,
     return false;
   }
 
-  const std::string& stringValue = args[1];
-  const std::string& variableName = args[2];
+  std::string const& stringValue = args[1];
+  std::string const& variableName = args[2];
   size_t inStringLength = stringValue.size();
   size_t startPos = inStringLength + 1;
   size_t endPos = 0;
-  const char* ptr = stringValue.c_str();
+  char const* ptr = stringValue.c_str();
   size_t cc;
   for (cc = 0; cc < inStringLength; ++cc) {
-    if (!isspace(*ptr)) {
+    if (!cmIsSpace(*ptr)) {
       if (startPos > inStringLength) {
         startPos = cc;
       }
@@ -718,9 +744,9 @@ bool HandleRepeatCommand(std::vector<std::string> const& args,
     return true;
   }
 
-  const auto& stringValue = args[ArgPos::VALUE];
-  const auto& variableName = args[ArgPos::OUTPUT_VARIABLE];
-  const auto inStringLength = stringValue.size();
+  auto const& stringValue = args[ArgPos::VALUE];
+  auto const& variableName = args[ArgPos::OUTPUT_VARIABLE];
+  auto const inStringLength = stringValue.size();
 
   std::string result;
   switch (inStringLength) {
@@ -757,7 +783,7 @@ bool HandleRandomCommand(std::vector<std::string> const& args,
   bool force_seed = false;
   unsigned int seed = 0;
   int length = 5;
-  const char cmStringCommandDefaultAlphabet[] = "qwertyuiopasdfghjklzxcvbnm"
+  char const cmStringCommandDefaultAlphabet[] = "qwertyuiopasdfghjklzxcvbnm"
                                                 "QWERTYUIOPASDFGHJKLZXCVBNM"
                                                 "0123456789";
   std::string alphabet;
@@ -793,7 +819,7 @@ bool HandleRandomCommand(std::vector<std::string> const& args,
     status.SetError("sub-command RANDOM invoked with bad length.");
     return false;
   }
-  const std::string& variableName = args.back();
+  std::string const& variableName = args.back();
 
   std::vector<char> result;
 
@@ -802,7 +828,7 @@ bool HandleRandomCommand(std::vector<std::string> const& args,
     srand(force_seed ? seed : cmSystemTools::RandomSeed());
   }
 
-  const char* alphaPtr = alphabet.c_str();
+  char const* alphaPtr = alphabet.c_str();
   for (int cc = 0; cc < length; cc++) {
     int idx = static_cast<int>(sizeofAlphabet * rand() / (RAND_MAX + 1.0));
     result.push_back(*(alphaPtr + idx));
@@ -827,7 +853,7 @@ bool HandleTimestampCommand(std::vector<std::string> const& args,
 
   unsigned int argsIndex = 1;
 
-  const std::string& outputVariable = args[argsIndex++];
+  std::string const& outputVariable = args[argsIndex++];
 
   std::string formatString;
   if (args.size() > argsIndex && args[argsIndex] != "UTC") {
@@ -864,7 +890,7 @@ bool HandleUuidCommand(std::vector<std::string> const& args,
     return false;
   }
 
-  const std::string& outputVariable = args[argsIndex++];
+  std::string const& outputVariable = args[argsIndex++];
 
   std::string uuidNamespaceString;
   std::string uuidName;
@@ -947,16 +973,16 @@ struct Args : cmRange<typename std::vector<std::string>::const_iterator>
 {
   using cmRange<typename std::vector<std::string>::const_iterator>::cmRange;
 
-  auto PopFront(cm::string_view error) -> const std::string&;
-  auto PopBack(cm::string_view error) -> const std::string&;
+  auto PopFront(cm::string_view error) -> std::string const&;
+  auto PopBack(cm::string_view error) -> std::string const&;
 };
 
 class json_error : public std::runtime_error
 {
 public:
-  json_error(std::initializer_list<cm::string_view> message,
+  json_error(std::string const& message,
              cm::optional<Args> errorPath = cm::nullopt)
-    : std::runtime_error(cmCatViews(message))
+    : std::runtime_error(message)
     , ErrorPath{
       std::move(errorPath) // NOLINT(performance-move-const-arg)
     }
@@ -965,22 +991,22 @@ public:
   cm::optional<Args> ErrorPath;
 };
 
-const std::string& Args::PopFront(cm::string_view error)
+std::string const& Args::PopFront(cm::string_view error)
 {
   if (this->empty()) {
-    throw json_error({ error });
+    throw json_error(std::string(error));
   }
-  const std::string& res = *this->begin();
+  std::string const& res = *this->begin();
   this->advance(1);
   return res;
 }
 
-const std::string& Args::PopBack(cm::string_view error)
+std::string const& Args::PopBack(cm::string_view error)
 {
   if (this->empty()) {
-    throw json_error({ error });
+    throw json_error(std::string(error));
   }
-  const std::string& res = *(this->end() - 1);
+  std::string const& res = *(this->end() - 1);
   this->retreat(1);
   return res;
 }
@@ -1003,23 +1029,23 @@ cm::string_view JsonTypeToString(Json::ValueType type)
     case Json::ValueType::objectValue:
       return "OBJECT"_s;
   }
-  throw json_error({ "invalid JSON type found"_s });
+  throw json_error("invalid JSON type found");
 }
 
 int ParseIndex(
-  const std::string& str, cm::optional<Args> const& progress = cm::nullopt,
+  std::string const& str, cm::optional<Args> const& progress = cm::nullopt,
   Json::ArrayIndex max = std::numeric_limits<Json::ArrayIndex>::max())
 {
   unsigned long lindex;
   if (!cmStrToULong(str, &lindex)) {
-    throw json_error({ "expected an array index, got: '"_s, str, "'"_s },
+    throw json_error(cmStrCat("expected an array index, got: '"_s, str, "'"_s),
                      progress);
   }
   Json::ArrayIndex index = static_cast<Json::ArrayIndex>(lindex);
   if (index >= max) {
     cmAlphaNum sizeStr{ max };
-    throw json_error({ "expected an index less then "_s, sizeStr.View(),
-                       " got '"_s, str, "'"_s },
+    throw json_error(cmStrCat("expected an index less than "_s, sizeStr.View(),
+                              " got '"_s, str, "'"_s),
                      progress);
   }
   return index;
@@ -1030,7 +1056,7 @@ Json::Value& ResolvePath(Json::Value& json, Args path)
   Json::Value* search = &json;
 
   for (auto curr = path.begin(); curr != path.end(); ++curr) {
-    const std::string& field = *curr;
+    std::string const& field = *curr;
     Args progress{ path.begin(), curr + 1 };
 
     if (search->isArray()) {
@@ -1039,24 +1065,24 @@ Json::Value& ResolvePath(Json::Value& json, Args path)
 
     } else if (search->isObject()) {
       if (!search->isMember(field)) {
-        const auto progressStr = cmJoin(progress, " "_s);
-        throw json_error({ "member '"_s, progressStr, "' not found"_s },
+        auto const progressStr = cmJoin(progress, " "_s);
+        throw json_error(cmStrCat("member '"_s, progressStr, "' not found"_s),
                          progress);
       }
       search = &(*search)[field];
     } else {
-      const auto progressStr = cmJoin(progress, " "_s);
+      auto const progressStr = cmJoin(progress, " "_s);
       throw json_error(
-        { "invalid path '"_s, progressStr,
-          "', need element of OBJECT or ARRAY type to lookup '"_s, field,
-          "' got "_s, JsonTypeToString(search->type()) },
+        cmStrCat("invalid path '"_s, progressStr,
+                 "', need element of OBJECT or ARRAY type to lookup '"_s,
+                 field, "' got "_s, JsonTypeToString(search->type())),
         progress);
     }
   }
   return *search;
-};
+}
 
-Json::Value ReadJson(const std::string& jsonstr)
+Json::Value ReadJson(std::string const& jsonstr)
 {
   Json::CharReaderBuilder builder;
   builder["collectComments"] = false;
@@ -1065,11 +1091,12 @@ Json::Value ReadJson(const std::string& jsonstr)
   std::string error;
   if (!jsonReader->parse(jsonstr.data(), jsonstr.data() + jsonstr.size(),
                          &json, &error)) {
-    throw json_error({ "failed parsing json string: "_s, error });
+    throw json_error(
+      cmStrCat("failed parsing json string:\n"_s, jsonstr, '\n', error));
   }
   return json;
 }
-std::string WriteJson(const Json::Value& value)
+std::string WriteJson(Json::Value const& value)
 {
   Json::StreamWriterBuilder writer;
   writer["indentation"] = "  ";
@@ -1087,8 +1114,8 @@ bool HandleJSONCommand(std::vector<std::string> const& arguments,
   auto& makefile = status.GetMakefile();
   Args args{ arguments.begin() + 1, arguments.end() };
 
-  const std::string* errorVariable = nullptr;
-  const std::string* outputVariable = nullptr;
+  std::string const* errorVariable = nullptr;
+  std::string const* outputVariable = nullptr;
   bool success = true;
 
   try {
@@ -1100,21 +1127,21 @@ bool HandleJSONCommand(std::vector<std::string> const& arguments,
       makefile.AddDefinition(*errorVariable, "NOTFOUND"_s);
     }
 
-    const auto& mode = args.PopFront("missing mode argument"_s);
+    auto const& mode = args.PopFront("missing mode argument"_s);
     if (mode != "GET"_s && mode != "TYPE"_s && mode != "MEMBER"_s &&
         mode != "LENGTH"_s && mode != "REMOVE"_s && mode != "SET"_s &&
         mode != "EQUAL"_s) {
       throw json_error(
-        { "got an invalid mode '"_s, mode,
-          "', expected one of GET, GET_ARRAY, TYPE, MEMBER, MEMBERS,"
-          " LENGTH, REMOVE, SET, EQUAL"_s });
+        cmStrCat("got an invalid mode '"_s, mode,
+                 "', expected one of GET, TYPE, MEMBER, LENGTH, REMOVE, SET, "
+                 " EQUAL"_s));
     }
 
-    const auto& jsonstr = args.PopFront("missing json string argument"_s);
+    auto const& jsonstr = args.PopFront("missing json string argument"_s);
     Json::Value json = ReadJson(jsonstr);
 
     if (mode == "GET"_s) {
-      const auto& value = ResolvePath(json, args);
+      auto const& value = ResolvePath(json, args);
       if (value.isObject() || value.isArray()) {
         makefile.AddDefinition(*outputVariable, WriteJson(value));
       } else if (value.isBool()) {
@@ -1124,29 +1151,30 @@ bool HandleJSONCommand(std::vector<std::string> const& arguments,
       }
 
     } else if (mode == "TYPE"_s) {
-      const auto& value = ResolvePath(json, args);
+      auto const& value = ResolvePath(json, args);
       makefile.AddDefinition(*outputVariable, JsonTypeToString(value.type()));
 
     } else if (mode == "MEMBER"_s) {
-      const auto& indexStr = args.PopBack("missing member index"_s);
-      const auto& value = ResolvePath(json, args);
+      auto const& indexStr = args.PopBack("missing member index"_s);
+      auto const& value = ResolvePath(json, args);
       if (!value.isObject()) {
-        throw json_error({ "MEMBER needs to be called with an element of "
-                           "type OBJECT, got "_s,
-                           JsonTypeToString(value.type()) },
-                         args);
+        throw json_error(
+          cmStrCat("MEMBER needs to be called with an element of "
+                   "type OBJECT, got "_s,
+                   JsonTypeToString(value.type())),
+          args);
       }
-      const auto index = ParseIndex(
+      auto const index = ParseIndex(
         indexStr, Args{ args.begin(), args.end() + 1 }, value.size());
-      const auto memIt = std::next(value.begin(), index);
+      auto const memIt = std::next(value.begin(), index);
       makefile.AddDefinition(*outputVariable, memIt.name());
 
     } else if (mode == "LENGTH"_s) {
-      const auto& value = ResolvePath(json, args);
+      auto const& value = ResolvePath(json, args);
       if (!value.isArray() && !value.isObject()) {
-        throw json_error({ "LENGTH needs to be called with an "
-                           "element of type ARRAY or OBJECT, got "_s,
-                           JsonTypeToString(value.type()) },
+        throw json_error(cmStrCat("LENGTH needs to be called with an "
+                                  "element of type ARRAY or OBJECT, got "_s,
+                                  JsonTypeToString(value.type())),
                          args);
       }
 
@@ -1154,12 +1182,12 @@ bool HandleJSONCommand(std::vector<std::string> const& arguments,
       makefile.AddDefinition(*outputVariable, sizeStr.View());
 
     } else if (mode == "REMOVE"_s) {
-      const auto& toRemove =
+      auto const& toRemove =
         args.PopBack("missing member or index to remove"_s);
       auto& value = ResolvePath(json, args);
 
       if (value.isArray()) {
-        const auto index = ParseIndex(
+        auto const index = ParseIndex(
           toRemove, Args{ args.begin(), args.end() + 1 }, value.size());
         Json::Value removed;
         value.removeIndex(index, &removed);
@@ -1169,23 +1197,23 @@ bool HandleJSONCommand(std::vector<std::string> const& arguments,
         value.removeMember(toRemove, &removed);
 
       } else {
-        throw json_error({ "REMOVE needs to be called with an "
-                           "element of type ARRAY or OBJECT, got "_s,
-                           JsonTypeToString(value.type()) },
+        throw json_error(cmStrCat("REMOVE needs to be called with an "
+                                  "element of type ARRAY or OBJECT, got "_s,
+                                  JsonTypeToString(value.type())),
                          args);
       }
       makefile.AddDefinition(*outputVariable, WriteJson(json));
 
     } else if (mode == "SET"_s) {
-      const auto& newValueStr = args.PopBack("missing new value remove"_s);
-      const auto& toAdd = args.PopBack("missing member name to add"_s);
+      auto const& newValueStr = args.PopBack("missing new value remove"_s);
+      auto const& toAdd = args.PopBack("missing member name to add"_s);
       auto& value = ResolvePath(json, args);
 
       Json::Value newValue = ReadJson(newValueStr);
       if (value.isObject()) {
         value[toAdd] = newValue;
       } else if (value.isArray()) {
-        const auto index =
+        auto const index =
           ParseIndex(toAdd, Args{ args.begin(), args.end() + 1 });
         if (value.isValidIndex(index)) {
           value[static_cast<int>(index)] = newValue;
@@ -1193,25 +1221,25 @@ bool HandleJSONCommand(std::vector<std::string> const& arguments,
           value.append(newValue);
         }
       } else {
-        throw json_error({ "SET needs to be called with an "
-                           "element of type OBJECT or ARRAY, got "_s,
-                           JsonTypeToString(value.type()) });
+        throw json_error(cmStrCat("SET needs to be called with an "
+                                  "element of type OBJECT or ARRAY, got "_s,
+                                  JsonTypeToString(value.type())));
       }
 
       makefile.AddDefinition(*outputVariable, WriteJson(json));
 
     } else if (mode == "EQUAL"_s) {
-      const auto& jsonstr2 =
+      auto const& jsonstr2 =
         args.PopFront("missing second json string argument"_s);
       Json::Value json2 = ReadJson(jsonstr2);
       makefile.AddDefinitionBool(*outputVariable, json == json2);
     }
 
-  } catch (const json_error& e) {
+  } catch (json_error const& e) {
     if (outputVariable && e.ErrorPath) {
-      const auto errorPath = cmJoin(*e.ErrorPath, "-");
+      auto const errorPath = cmJoin(*e.ErrorPath, "-");
       makefile.AddDefinition(*outputVariable,
-                             cmCatViews({ errorPath, "-NOTFOUND"_s }));
+                             cmStrCat(errorPath, "-NOTFOUND"_s));
     } else if (outputVariable) {
       makefile.AddDefinition(*outputVariable, "NOTFOUND"_s);
     }
@@ -1219,7 +1247,7 @@ bool HandleJSONCommand(std::vector<std::string> const& arguments,
     if (errorVariable) {
       makefile.AddDefinition(*errorVariable, e.what());
     } else {
-      status.SetError(cmCatViews({ "sub-command JSON "_s, e.what(), "."_s }));
+      status.SetError(cmStrCat("sub-command JSON "_s, e.what(), "."_s));
       success = false;
     }
   }
